@@ -1,15 +1,13 @@
 // -*- mode: js; js-indent-level: 4; indent-tabs-mode: nil -*-
-/* exported CredentialManager */
 
-const Signals = imports.misc.signals;
+import * as Signals from '../misc/signals.js';
 
-var CredentialManager = class CredentialManager extends Signals.EventEmitter {
+export class CredentialManager extends Signals.EventEmitter {
     constructor(service) {
         super();
 
         this._token = null;
         this._service = service;
-        this._authenticatedSignalId = null;
     }
 
     get token() {
@@ -25,4 +23,4 @@ var CredentialManager = class CredentialManager extends Signals.EventEmitter {
     get service() {
         return this._service;
     }
-};
+}
