@@ -1,13 +1,12 @@
 // -*- mode: js; js-indent-level: 4; indent-tabs-mode: nil -*-
-/* exported getVmwareCredentialsManager */
 
-const Gio = imports.gi.Gio;
-const Credential = imports.gdm.credentialManager;
+import Gio from 'gi://Gio';
+import * as Credential from './credentialManager.js';
 
 const dbusPath = '/org/vmware/viewagent/Credentials';
 const dbusInterface = 'org.vmware.viewagent.Credentials';
 
-var SERVICE_NAME = 'gdm-vmwcred';
+export const SERVICE_NAME = 'gdm-vmwcred';
 
 const VmwareCredentialsIface = `<node>
 <interface name="${dbusInterface}">
@@ -35,7 +34,7 @@ function VmwareCredentials() {
     return self;
 }
 
-var VmwareCredentialsManager = class VmwareCredentialsManager extends Credential.CredentialManager {
+class VmwareCredentialsManager extends Credential.CredentialManager {
     constructor() {
         super(SERVICE_NAME);
         this._credentials = new VmwareCredentials();
@@ -44,9 +43,12 @@ var VmwareCredentialsManager = class VmwareCredentialsManager extends Credential
                 this.token = token;
             });
     }
-};
+}
 
-function getVmwareCredentialsManager() {
+/**
+ * @returns {VmwareCredentialsManager}
+ */
+export function getVmwareCredentialsManager() {
     if (!_vmwareCredentialsManager)
         _vmwareCredentialsManager = new VmwareCredentialsManager();
 
