@@ -1,23 +1,26 @@
 // -*- mode: js; js-indent-level: 4; indent-tabs-mode: nil -*-
-/* exported InputMethod */
-const { Clutter, GLib, Gio, GObject, IBus } = imports.gi;
 
-const Keyboard = imports.ui.status.keyboard;
-const Main = imports.ui.main;
+import Clutter from 'gi://Clutter';
+import GLib from 'gi://GLib';
+import Gio from 'gi://Gio';
+import GObject from 'gi://GObject';
+import IBus from 'gi://IBus';
+
+import * as Keyboard from '../ui/status/keyboard.js';
+import * as Main from '../ui/main.js';
 
 Gio._promisify(IBus.Bus.prototype,
     'create_input_context_async', 'create_input_context_async_finish');
 Gio._promisify(IBus.InputContext.prototype,
     'process_key_event_async', 'process_key_event_async_finish');
 
-var HIDE_PANEL_TIME = 50;
+const HIDE_PANEL_TIME = 50;
 
 const HAVE_REQUIRE_SURROUNDING_TEXT = GObject.signal_lookup('require-surrounding-text', IBus.InputContext);
 
-var InputMethod = GObject.registerClass({
+export const InputMethod = GObject.registerClass({
     Signals: {
         'surrounding-text-set': {},
-        'terminal-mode-changed': {},
     },
 }, class InputMethod extends Clutter.InputMethod {
     _init() {
@@ -30,7 +33,6 @@ var InputMethod = GObject.registerClass({
         this._preeditAnchor = 0;
         this._preeditVisible = false;
         this._hidePanelId = 0;
-        this.terminalMode = false;
         this._ibus = IBus.Bus.new_async();
         this._ibus.connect('connected', this._onConnected.bind(this));
         this._ibus.connect('disconnected', this._clear.bind(this));
@@ -38,7 +40,7 @@ var InputMethod = GObject.registerClass({
 
         this._inputSourceManager = Keyboard.getInputSourceManager();
         this._sourceChangedId = this._inputSourceManager.connect('current-source-changed',
-                                                                 this._onSourceChanged.bind(this));
+            this._onSourceChanged.bind(this));
         this._currentSource = this._inputSourceManager.currentSource;
 
         if (this._ibus.is_connected())
@@ -168,7 +170,7 @@ var InputMethod = GObject.registerClass({
     }
 
     _onForwardKeyEvent(_context, keyval, keycode, state) {
-        let press = (state & IBus.ModifierType.RELEASE_MASK) == 0;
+        let press = (state & IBus.ModifierType.RELEASE_MASK) === 0;
         state &= ~IBus.ModifierType.RELEASE_MASK;
 
         let curEvent = Clutter.get_current_event();
@@ -223,8 +225,8 @@ var InputMethod = GObject.registerClass({
 
         this._surroundingText = null;
         this._surroundingTextCursor = null;
+        this._surroundingTextAnchor = null;
         this._preeditStr = null;
-        this._setTerminalMode(false);
     }
 
     vfunc_set_cursor_location(rect) {
@@ -243,6 +245,7 @@ var InputMethod = GObject.registerClass({
     vfunc_set_surrounding(text, cursor, anchor) {
         this._surroundingText = text;
         this._surroundingTextCursor = cursor;
+        this._surroundingTextAnchor = anchor;
         this.emit('surrounding-text-set');
 
         if (!this._context || (!text && text !== ''))
@@ -266,6 +269,8 @@ var InputMethod = GObject.registerClass({
             ibusHints |= IBus.InputHints.UPPERCASE_CHARS;
         if (hints & Clutter.InputContentHintFlags.TITLECASE)
             ibusHints |= IBus.InputHints.UPPERCASE_WORDS;
+        if (hints & Clutter.InputContentHintFlags.SENSITIVE_DATA)
+            ibusHints |= IBus.InputHints.PRIVATE;
 
         this._hints = ibusHints;
         if (this._context)
@@ -274,43 +279,33 @@ var InputMethod = GObject.registerClass({
 
     vfunc_update_content_purpose(purpose) {
         let ibusPurpose = 0;
-        if (purpose == Clutter.InputContentPurpose.NORMAL)
+        if (purpose === Clutter.InputContentPurpose.NORMAL)
             ibusPurpose = IBus.InputPurpose.FREE_FORM;
-        else if (purpose == Clutter.InputContentPurpose.ALPHA)
+        else if (purpose === Clutter.InputContentPurpose.ALPHA)
             ibusPurpose = IBus.InputPurpose.ALPHA;
-        else if (purpose == Clutter.InputContentPurpose.DIGITS)
+        else if (purpose === Clutter.InputContentPurpose.DIGITS)
             ibusPurpose = IBus.InputPurpose.DIGITS;
-        else if (purpose == Clutter.InputContentPurpose.NUMBER)
+        else if (purpose === Clutter.InputContentPurpose.NUMBER)
             ibusPurpose = IBus.InputPurpose.NUMBER;
-        else if (purpose == Clutter.InputContentPurpose.PHONE)
+        else if (purpose === Clutter.InputContentPurpose.PHONE)
             ibusPurpose = IBus.InputPurpose.PHONE;
-        else if (purpose == Clutter.InputContentPurpose.URL)
+        else if (purpose === Clutter.InputContentPurpose.URL)
             ibusPurpose = IBus.InputPurpose.URL;
-        else if (purpose == Clutter.InputContentPurpose.EMAIL)
+        else if (purpose === Clutter.InputContentPurpose.EMAIL)
             ibusPurpose = IBus.InputPurpose.EMAIL;
-        else if (purpose == Clutter.InputContentPurpose.NAME)
+        else if (purpose === Clutter.InputContentPurpose.NAME)
             ibusPurpose = IBus.InputPurpose.NAME;
-        else if (purpose == Clutter.InputContentPurpose.PASSWORD)
+        else if (purpose === Clutter.InputContentPurpose.PASSWORD)
             ibusPurpose = IBus.InputPurpose.PASSWORD;
         else if (purpose === Clutter.InputContentPurpose.TERMINAL &&
                  IBus.InputPurpose.TERMINAL)
             ibusPurpose = IBus.InputPurpose.TERMINAL;
 
-        this._setTerminalMode(
-            purpose === Clutter.InputContentPurpose.TERMINAL);
-
         this._purpose = ibusPurpose;
         if (this._context)
             this._context.set_content_type(this._purpose, this._hints);
     }
 
-    _setTerminalMode(terminalMode) {
-        if (this.terminalMode !== terminalMode) {
-            this.terminalMode = terminalMode;
-            this.emit('terminal-mode-changed');
-        }
-    }
-
     vfunc_filter_key_event(event) {
         if (!this._context)
             return false;
@@ -321,7 +316,7 @@ var InputMethod = GObject.registerClass({
         if (state & IBus.ModifierType.IGNORED_MASK)
             return false;
 
-        if (event.type() == Clutter.EventType.KEY_RELEASE)
+        if (event.type() === Clutter.EventType.KEY_RELEASE)
             state |= IBus.ModifierType.RELEASE_MASK;
 
         this._context.process_key_event_async(
@@ -329,7 +324,7 @@ var InputMethod = GObject.registerClass({
             event.get_key_code() - 8, // Convert XKB keycodes to evcodes
             state, -1, this._cancellable,
             (context, res) => {
-                if (context != this._context)
+                if (context !== this._context)
                     return;
 
                 try {
@@ -344,7 +339,11 @@ var InputMethod = GObject.registerClass({
     }
 
     getSurroundingText() {
-        return [this._surroundingText, this._surroundingTextCursor];
+        return [
+            this._surroundingText,
+            this._surroundingTextCursor,
+            this._surroundingTextAnchor,
+        ];
     }
 
     hasPreedit() {
