@@ -1,10 +1,14 @@
-/* exported getDefault */
-const { AccountsService, Clutter, Gdm, Gio, GLib, GObject, Meta } = imports.gi;
+import AccountsService from 'gi://AccountsService';
+import Clutter from 'gi://Clutter';
+import Gdm from 'gi://Gdm';
+import Gio from 'gi://Gio';
+import GLib from 'gi://GLib';
+import GObject from 'gi://GObject';
 
-const GnomeSession = imports.misc.gnomeSession;
-const LoginManager = imports.misc.loginManager;
-const Main = imports.ui.main;
-const Screenshot = imports.ui.screenshot;
+import * as GnomeSession from './gnomeSession.js';
+import * as LoginManager from './loginManager.js';
+import * as Main from '../ui/main.js';
+import * as Screenshot from '../ui/screenshot.js';
 
 const LOCKDOWN_SCHEMA = 'org.gnome.desktop.lockdown';
 const LOGIN_SCREEN_SCHEMA = 'org.gnome.login-screen';
@@ -25,7 +29,10 @@ const SCREENSHOT_UI_ACTION_ID    = 'open-screenshot-ui';
 
 let _singleton = null;
 
-function getDefault() {
+/**
+ * @returns {SystemActions}
+ */
+export function getDefault() {
     if (_singleton == null)
         _singleton = new SystemActions();
 
@@ -81,10 +88,10 @@ const SystemActions = GObject.registerClass({
         this._actions = new Map();
         this._actions.set(POWER_OFF_ACTION_ID, {
             // Translators: The name of the power-off action in search
-            name: C_("search-result", "Power Off"),
+            name: C_('search-result', 'Power Off'),
             iconName: 'system-shutdown-symbolic',
             // Translators: A list of keywords that match the power-off action, separated by semicolons
-            keywords: tokenizeKeywords(_('power off;shutdown;halt;stop')),
+            keywords: tokenizeKeywords(_('power off;poweroff;shutdown;halt;stop')),
             available: false,
         });
         this._actions.set(RESTART_ACTION_ID, {
@@ -97,7 +104,7 @@ const SystemActions = GObject.registerClass({
         });
         this._actions.set(LOCK_SCREEN_ACTION_ID, {
             // Translators: The name of the lock screen action in search
-            name: C_("search-result", "Lock Screen"),
+            name: C_('search-result', 'Lock Screen'),
             iconName: 'system-lock-screen-symbolic',
             // Translators: A list of keywords that match the lock screen action, separated by semicolons
             keywords: tokenizeKeywords(_('lock screen')),
@@ -105,7 +112,7 @@ const SystemActions = GObject.registerClass({
         });
         this._actions.set(LOGOUT_ACTION_ID, {
             // Translators: The name of the logout action in search
-            name: C_("search-result", "Log Out"),
+            name: C_('search-result', 'Log Out'),
             iconName: 'system-log-out-symbolic',
             // Translators: A list of keywords that match the logout action, separated by semicolons
             keywords: tokenizeKeywords(_('logout;log out;sign off')),
@@ -113,7 +120,7 @@ const SystemActions = GObject.registerClass({
         });
         this._actions.set(SUSPEND_ACTION_ID, {
             // Translators: The name of the suspend action in search
-            name: C_("search-result", "Suspend"),
+            name: C_('search-result', 'Suspend'),
             iconName: 'media-playback-pause-symbolic',
             // Translators: A list of keywords that match the suspend action, separated by semicolons
             keywords: tokenizeKeywords(_('suspend;sleep')),
@@ -121,7 +128,7 @@ const SystemActions = GObject.registerClass({
         });
         this._actions.set(SWITCH_USER_ACTION_ID, {
             // Translators: The name of the switch user action in search
-            name: C_("search-result", "Switch User"),
+            name: C_('search-result', 'Switch User'),
             iconName: 'system-switch-user-symbolic',
             // Translators: A list of keywords that match the switch user action, separated by semicolons
             keywords: tokenizeKeywords(_('switch user')),
@@ -143,37 +150,37 @@ const SystemActions = GObject.registerClass({
             available: true,
         });
 
-        this._loginScreenSettings = new Gio.Settings({ schema_id: LOGIN_SCREEN_SCHEMA });
-        this._lockdownSettings = new Gio.Settings({ schema_id: LOCKDOWN_SCHEMA });
-        this._orientationSettings = new Gio.Settings({ schema_id: 'org.gnome.settings-daemon.peripherals.touchscreen' });
+        this._loginScreenSettings = new Gio.Settings({schema_id: LOGIN_SCREEN_SCHEMA});
+        this._lockdownSettings = new Gio.Settings({schema_id: LOCKDOWN_SCHEMA});
+        this._orientationSettings = new Gio.Settings({schema_id: 'org.gnome.settings-daemon.peripherals.touchscreen'});
 
         this._session = new GnomeSession.SessionManager();
         this._loginManager = LoginManager.getLoginManager();
-        this._monitorManager = Meta.MonitorManager.get();
+        this._monitorManager = global.backend.get_monitor_manager();
 
         this._userManager = AccountsService.UserManager.get_default();
 
         this._userManager.connect('notify::is-loaded',
-                                  () => this._updateMultiUser());
+            () => this._updateMultiUser());
         this._userManager.connect('notify::has-multiple-users',
-                                  () => this._updateMultiUser());
+            () => this._updateMultiUser());
         this._userManager.connect('user-added',
-                                  () => this._updateMultiUser());
+            () => this._updateMultiUser());
         this._userManager.connect('user-removed',
-                                  () => this._updateMultiUser());
+            () => this._updateMultiUser());
 
         this._lockdownSettings.connect(`changed::${DISABLE_USER_SWITCH_KEY}`,
-                                       () => this._updateSwitchUser());
+            () => this._updateSwitchUser());
         this._lockdownSettings.connect(`changed::${DISABLE_LOG_OUT_KEY}`,
-                                       () => this._updateLogout());
+            () => this._updateLogout());
         global.settings.connect(`changed::${ALWAYS_SHOW_LOG_OUT_KEY}`,
-                                () => this._updateLogout());
+            () => this._updateLogout());
 
         this._lockdownSettings.connect(`changed::${DISABLE_LOCK_SCREEN_KEY}`,
-                                       () => this._updateLockScreen());
+            () => this._updateLockScreen());
 
         this._lockdownSettings.connect(`changed::${DISABLE_LOG_OUT_KEY}`,
-                                       () => this._updateHaveShutdown());
+            () => this._updateHaveShutdown());
 
         this.forceUpdate();
 
@@ -182,7 +189,7 @@ const SystemActions = GObject.registerClass({
             this._updateOrientationLockStatus();
         });
         Main.layoutManager.connect('monitors-changed',
-                                   () => this._updateOrientationLock());
+            () => this._updateOrientationLock());
         this._monitorManager.connect('notify::panel-orientation-managed',
             () => this._updateOrientationLock());
         this._updateOrientationLock();
@@ -277,7 +284,7 @@ const SystemActions = GObject.registerClass({
 
         let results = [];
 
-        for (let [key, { available, keywords }] of this._actions) {
+        for (let [key, {available, keywords}] of this._actions) {
             if (available && terms.every(t => keywords.some(k => k.startsWith(t))))
                 results.push(key);
         }
