@@ -118,12 +118,19 @@ export function spawnApp(argv) {
  * this will throw an error.
  */
 export function trySpawn(argv) {
-    let pid;
+    let success_, pid;
     try {
         const launchContext = global.create_app_launch_context(0, -1);
-        pid = Shell.util_spawn_async(
+        [success_, pid] = GLib.spawn_async(
             null, argv, launchContext.get_environment(),
-            GLib.SpawnFlags.SEARCH_PATH | GLib.SpawnFlags.DO_NOT_REAP_CHILD);
+            GLib.SpawnFlags.SEARCH_PATH | GLib.SpawnFlags.DO_NOT_REAP_CHILD,
+            () => {
+                try {
+                    global.context.restore_rlimit_nofile();
+                } catch (err) {
+                }
+            }
+        );
     } catch (err) {
         /* Rewrite the error in case of ENOENT */
         if (err.matches(GLib.SpawnError, GLib.SpawnError.NOENT)) {
@@ -287,8 +294,10 @@ function _GNOMEversionToNumber(version) {
     if (!isNaN(ret))
         return ret;
     if (version === 'alpha')
-        return -2;
+        return -3;
     if (version === 'beta')
+        return -2;
+    if (version === 'rc')
         return -1;
     return ret;
 }
