@@ -21,14 +21,20 @@ class Animation extends St.Bin {
             style: `width: ${width}px; height: ${height}px;`,
         });
 
+        this._file = file;
+        this._width = width;
+        this._height = height;
+
         this.connect('destroy', this._onDestroy.bind(this));
         this.connect('resource-scale-changed',
-            this._loadFile.bind(this, file, width, height));
+            () => this._loadFile());
 
         themeContext.connectObject('notify::scale-factor',
             () => {
-                this._loadFile(file, width, height);
-                this.set_size(width * themeContext.scale_factor, height * themeContext.scale_factor);
+                this._loadFile();
+                this.set_size(
+                    this._width * themeContext.scale_factor,
+                    this._height * themeContext.scale_factor);
             }, this);
 
         this._speed = speed;
@@ -38,7 +44,7 @@ class Animation extends St.Bin {
         this._timeoutId = 0;
         this._frame = 0;
 
-        this._loadFile(file, width, height);
+        this._loadFile();
     }
 
     play() {
@@ -62,7 +68,7 @@ class Animation extends St.Bin {
         this._isPlaying = false;
     }
 
-    _loadFile(file, width, height) {
+    _loadFile() {
         const resourceScale = this.get_resource_scale();
         let wasPlaying = this._isPlaying;
 
@@ -74,7 +80,8 @@ class Animation extends St.Bin {
 
         let textureCache = St.TextureCache.get_default();
         let scaleFactor = St.ThemeContext.get_for_stage(global.stage).scale_factor;
-        this._animations = textureCache.load_sliced_image(file, width, height,
+        this._animations = textureCache.load_sliced_image(this._file,
+            this._width, this._height,
             scaleFactor, resourceScale,
             () => this._loadFinished());
         this._animations.set({
@@ -131,8 +138,24 @@ class Spinner extends AnimatedIcon {
             animate: false,
             hideOnStop: false,
         });
-        let file = Gio.File.new_for_uri('resource:///org/gnome/shell/theme/process-working.svg');
-        super._init(file, size);
+        this._fileDark = Gio.File.new_for_uri(
+            'resource:///org/gnome/shell/theme/process-working-dark.svg');
+        this._fileLight = Gio.File.new_for_uri(
+            'resource:///org/gnome/shell/theme/process-working-light.svg');
+        super._init(this._fileDark, size);
+
+        this.connect('style-changed', () => {
+            const themeNode = this.get_theme_node();
+            const textColor = themeNode.get_foreground_color();
+            const [, luminance] = textColor.to_hls();
+            const file = luminance > 0.5
+                ? this._fileDark
+                : this._fileLight;
+            if (file !== this._file) {
+                this._file = file;
+                this._loadFile();
+            }
+        });
 
         this.opacity = 0;
         this._animate = params.animate;
