@@ -21,13 +21,13 @@ class CheckBox extends St.Button {
         this.set_accessible_role(Atk.Role.CHECK_BOX);
 
         this._box = new St.Bin({y_align: Clutter.ActorAlign.START});
-        container.add_actor(this._box);
+        container.add_child(this._box);
 
         this._label = new St.Label({y_align: Clutter.ActorAlign.CENTER});
         this._label.clutter_text.set_line_wrap(true);
         this._label.clutter_text.set_ellipsize(Pango.EllipsizeMode.NONE);
         this.set_label_actor(this._label);
-        container.add_actor(this._label);
+        container.add_child(this._label);
 
         if (label)
             this.setLabel(label);
