@@ -11,7 +11,6 @@ import St from 'gi://St';
 import * as Signals from '../../misc/signals.js';
 
 import * as Dialog from '../dialog.js';
-import * as Main from '../main.js';
 import * as MessageTray from '../messageTray.js';
 import * as ModalDialog from '../modalDialog.js';
 import * as ShellEntry from '../shellEntry.js';
@@ -456,12 +455,18 @@ class VPNRequestHandler extends Signals.EventEmitter {
 
         try {
             const launchContext = global.create_app_launch_context(0, -1);
-            let [pid, stdin, stdout, stderr] =
-                Shell.util_spawn_async_with_pipes(
+            let [success_, pid, stdin, stdout, stderr] =
+                GLib.spawn_async_with_pipes(
                     null, /* pwd */
                     argv,
                     launchContext.get_environment(),
-                    GLib.SpawnFlags.DO_NOT_REAP_CHILD);
+                    GLib.SpawnFlags.DO_NOT_REAP_CHILD,
+                    () => {
+                        try {
+                            global.context.restore_rlimit_nofile();
+                        } catch (err) {
+                        }
+                    });
 
             this._childPid = pid;
             this._stdin = new Gio.UnixOutputStream({fd: stdin, close_fd: true});
@@ -643,7 +648,7 @@ class VPNRequestHandler extends Signals.EventEmitter {
         if (contentOverride && contentOverride.secrets.length) {
             // Only show the dialog if we actually have something to ask
             this._shellDialog = new NetworkSecretDialog(this._agent, this._requestId, this._connection, 'vpn', [], this._flags, contentOverride);
-            this._shellDialog.open(global.get_current_time());
+            this._shellDialog.open();
         } else {
             this._agent.respond(this._requestId, Shell.NetworkAgentResponse.CONFIRMED);
             this.destroy();
@@ -734,9 +739,6 @@ class NetworkAgent {
     }
 
     _showNotification(requestId, connection, settingName, hints, flags) {
-        let source = new MessageTray.Source(_('Network Manager'), 'network-transmit-receive');
-        source.policy = new MessageTray.NotificationApplicationPolicy('gnome-network-panel');
-
         let title, body;
 
         let connectionSetting = connection.get_setting_connection();
@@ -779,7 +781,9 @@ class NetworkAgent {
             return;
         }
 
-        let notification = new MessageTray.Notification(source, title, body);
+        const source = MessageTray.getSystemSource();
+        const notification = new MessageTray.Notification({source, title, body});
+        notification.iconName = 'dialog-password-symbolic';
 
         notification.connect('activated', () => {
             notification.answered = true;
@@ -793,8 +797,7 @@ class NetworkAgent {
             delete this._notifications[requestId];
         });
 
-        Main.messageTray.add(source);
-        source.showNotification(notification);
+        source.addNotification(notification);
     }
 
     _newRequest(agent, requestId, connection, settingName, hints, flags) {
@@ -815,7 +818,7 @@ class NetworkAgent {
             delete this._dialogs[requestId];
         });
         this._dialogs[requestId] = dialog;
-        dialog.open(global.get_current_time());
+        dialog.open();
     }
 
     _cancelRequest(agent, requestId) {
