@@ -63,17 +63,20 @@ class TodayButton extends St.Button {
             reactive: false,
         });
 
-        const hbox = new St.BoxLayout({vertical: true});
-        this.add_actor(hbox);
+        const hbox = new St.BoxLayout({
+            vertical: true,
+            x_expand: true,
+        });
+        this.child = hbox;
 
         this._dayLabel = new St.Label({
             style_class: 'day-label',
             x_align: Clutter.ActorAlign.START,
         });
-        hbox.add_actor(this._dayLabel);
+        hbox.add_child(this._dayLabel);
 
         this._dateLabel = new St.Label({style_class: 'date-label'});
-        hbox.add_actor(this._dateLabel);
+        hbox.add_child(this._dateLabel);
 
         this._calendar = calendar;
         this._calendar.connect('selected-date-changed', (_calendar, datetime) => {
@@ -272,11 +275,11 @@ class EventsSection extends St.Button {
                 style_class: 'event-box',
                 vertical: true,
             });
-            box.add(new St.Label({
+            box.add_child(new St.Label({
                 text: event.summary,
                 style_class: 'event-summary',
             }));
-            box.add(new St.Label({
+            box.add_child(new St.Label({
                 text: this._formatEventTime(event),
                 style_class: 'event-time',
             }));
@@ -414,6 +417,12 @@ class WorldClocksSection extends St.Button {
             x_align: Clutter.ActorAlign.START,
             text: title,
         });
+
+        if (this._locations.length === 0)
+            header.add_style_class_name('no-world-clocks');
+        else
+            header.remove_style_class_name('no-world-clocks');
+
         if (this._grid.text_direction === Clutter.TextDirection.RTL)
             layout.attach(header, 2, 0, 1, 1);
         else
@@ -718,6 +727,11 @@ class WeatherSection extends St.Button {
         else
             this._titleLabel.text = _('Select weather location…');
 
+        if (this._weatherClient.hasLocation)
+            this._titleLabel.remove_style_class_name('no-location');
+        else
+            this._titleLabel.add_style_class_name('no-location');
+
         this._forecastGrid.visible = this._weatherClient.hasLocation;
         this._titleLocation.visible = this._weatherClient.hasLocation;
 
@@ -729,7 +743,7 @@ const MessagesIndicator = GObject.registerClass(
 class MessagesIndicator extends St.Icon {
     _init() {
         super._init({
-            icon_size: 16,
+            style_class: 'messages-indicator',
             visible: false,
             y_expand: true,
             y_align: Clutter.ActorAlign.CENTER,
@@ -868,12 +882,12 @@ class DateMenuButton extends PanelMenu.Button {
         }));
 
         let box = new St.BoxLayout({style_class: 'clock-display-box'});
-        box.add_actor(indicatorPad);
-        box.add_actor(this._clockDisplay);
-        box.add_actor(this._indicator);
+        box.add_child(indicatorPad);
+        box.add_child(this._clockDisplay);
+        box.add_child(this._indicator);
 
         this.label_actor = this._clockDisplay;
-        this.add_actor(box);
+        this.add_child(box);
         this.add_style_class_name('clock-display');
 
         let layout = new FreezableBinLayout();
@@ -881,9 +895,10 @@ class DateMenuButton extends PanelMenu.Button {
         // For some minimal compatibility with PopupMenuItem
         bin._delegate = this;
         this.menu.box.add_child(bin);
+        this.menu.box.add_style_class_name('datemenu-popover');
 
         hbox = new St.BoxLayout({name: 'calendarArea'});
-        bin.add_actor(hbox);
+        bin.add_child(hbox);
 
         this._calendar = new Calendar.Calendar();
         this._calendar.connect('selected-date-changed', (_calendar, datetime) => {
@@ -914,25 +929,25 @@ class DateMenuButton extends PanelMenu.Button {
             layout_manager: boxLayout,
         });
         boxLayout.hookup_style(vbox);
-        hbox.add(vbox);
+        hbox.add_child(vbox);
 
-        vbox.add_actor(this._date);
-        vbox.add_actor(this._calendar);
-
-        this._displaysSection = new St.ScrollView({
-            style_class: 'datemenu-displays-section vfade',
-            x_expand: true,
-            overlay_scrollbars: true,
-        });
-        this._displaysSection.set_policy(St.PolicyType.NEVER, St.PolicyType.EXTERNAL);
-        vbox.add_actor(this._displaysSection);
+        vbox.add_child(this._date);
+        vbox.add_child(this._calendar);
 
         const displaysBox = new St.BoxLayout({
             vertical: true,
             x_expand: true,
             style_class: 'datemenu-displays-box',
         });
-        this._displaysSection.add_actor(displaysBox);
+
+        this._displaysSection = new St.ScrollView({
+            style_class: 'datemenu-displays-section vfade',
+            x_expand: true,
+            overlay_scrollbars: true,
+            vscrollbar_policy: St.PolicyType.EXTERNAL,
+            child: displaysBox,
+        });
+        vbox.add_child(this._displaysSection);
 
         this._eventsItem = new EventsSection();
         displaysBox.add_child(this._eventsItem);
