@@ -144,7 +144,7 @@ class Dialog extends St.Widget {
         for (let i in keys)
             this._buttonKeys[keys[i]] = buttonInfo;
 
-        this.buttonLayout.add_actor(button);
+        this.buttonLayout.add_child(button);
 
         return button;
     }
@@ -176,12 +176,12 @@ export const MessageDialogContent = GObject.registerClass({
         this._description.clutter_text.ellipsize = Pango.EllipsizeMode.NONE;
         this._description.clutter_text.line_wrap = true;
 
-        let defaultParams = {
+        super._init({
             style_class: 'message-dialog-content',
             x_expand: true,
             vertical: true,
-        };
-        super._init(Object.assign(defaultParams, params));
+            ...params,
+        });
 
         this.connect('notify::size', this._updateTitleStyle.bind(this));
         this.connect('destroy', this._onDestroy.bind(this));
@@ -259,23 +259,22 @@ export const ListSection = GObject.registerClass({
     _init(params) {
         this._title = new St.Label({style_class: 'dialog-list-title'});
 
-        this._listScrollView = new St.ScrollView({
-            style_class: 'dialog-list-scrollview',
-            hscrollbar_policy: St.PolicyType.NEVER,
-        });
-
         this.list = new St.BoxLayout({
             style_class: 'dialog-list-box',
             vertical: true,
         });
-        this._listScrollView.add_actor(this.list);
 
-        let defaultParams = {
+        this._listScrollView = new St.ScrollView({
+            style_class: 'dialog-list-scrollview',
+            child: this.list,
+        });
+
+        super._init({
             style_class: 'dialog-list',
             x_expand: true,
             vertical: true,
-        };
-        super._init(Object.assign(defaultParams, params));
+            ...params,
+        });
 
         this.label_actor = this._title;
         this.add_child(this._title);
@@ -328,8 +327,10 @@ export const ListSectionItem = GObject.registerClass({
         textLayout.add_child(this._title);
         textLayout.add_child(this._description);
 
-        let defaultParams = {style_class: 'dialog-list-item'};
-        super._init(Object.assign(defaultParams, params));
+        super._init({
+            style_class: 'dialog-list-item',
+            ...params,
+        });
 
         this.label_actor = this._title;
         this.add_child(this._iconActorBin);
