@@ -12,6 +12,7 @@ import * as Main from './main.js';
 import * as MessageTray from './messageTray.js';
 
 import * as Util from '../misc/util.js';
+import {formatTimeSpan} from '../misc/dateUtils.js';
 
 const MESSAGE_ANIMATION_TIME = 100;
 
@@ -56,7 +57,7 @@ class URLHighlighter extends St.Label {
         this.connect('style-changed', () => {
             let [hasColor, color] = this.get_theme_node().lookup_color('link-color', false);
             if (hasColor) {
-                let linkColor = color.to_string().substring(0, 7);
+                let linkColor = color.to_string().substr(0, 7);
                 if (linkColor !== this._linkColor) {
                     this._linkColor = linkColor;
                     this._highlightUrls();
@@ -142,11 +143,11 @@ class URLHighlighter extends St.Label {
         let pos = 0;
         for (let i = 0; i < urls.length; i++) {
             let url = urls[i];
-            let str = this._text.substring(pos, url.pos);
+            let str = this._text.substr(pos, url.pos - pos);
             markup += `${str}<span foreground="${this._linkColor}"><u>${url.url}</u></span>`;
             pos = url.pos + url.url.length;
         }
-        markup += this._text.substring(pos);
+        markup += this._text.substr(pos);
         this.clutter_text.set_markup(markup);
     }
 
@@ -221,12 +222,12 @@ const LabelExpanderLayout = GObject.registerClass({
             GObject.ParamFlags.READABLE | GObject.ParamFlags.WRITABLE,
             0, 1, 0),
     },
-}, class LabelExpanderLayout extends Clutter.LayoutManager {
-    _init(params) {
+}, class LabelExpanderLayout extends Clutter.BinLayout {
+    constructor(params) {
+        super(params);
+
         this._expansion = 0;
         this._expandLines = DEFAULT_EXPAND_LINES;
-
-        super._init(params);
     }
 
     get expansion() {
@@ -239,10 +240,6 @@ const LabelExpanderLayout = GObject.registerClass({
         this._expansion = v;
         this.notify('expansion');
 
-        let visibleIndex = this._expansion > 0 ? 1 : 0;
-        for (let i = 0; this._container && i < this._container.get_n_children(); i++)
-            this._container.get_child_at_index(i).visible = i === visibleIndex;
-
         this.layout_changed();
     }
 
@@ -254,72 +251,200 @@ const LabelExpanderLayout = GObject.registerClass({
             this.layout_changed();
     }
 
-    vfunc_set_container(container) {
-        this._container = container;
-    }
-
-    vfunc_get_preferred_width(container, forHeight) {
-        let [min, nat] = [0, 0];
-
-        for (let i = 0; i < container.get_n_children(); i++) {
-            if (i > 1)
-                break; // we support one unexpanded + one expanded child
-
-            let child = container.get_child_at_index(i);
-            let [childMin, childNat] = child.get_preferred_width(forHeight);
-            [min, nat] = [Math.max(min, childMin), Math.max(nat, childNat)];
-        }
-
-        return [min, nat];
-    }
-
     vfunc_get_preferred_height(container, forWidth) {
         let [min, nat] = [0, 0];
 
-        let children = container.get_children();
-        if (children[0])
-            [min, nat] = children[0].get_preferred_height(forWidth);
+        const [child] = container;
 
-        if (children[1]) {
-            let [min2, nat2] = children[1].get_preferred_height(forWidth);
-            const [expMin, expNat] = [
-                Math.min(min2, min * this._expandLines),
-                Math.min(nat2, nat * this._expandLines),
-            ];
+        if (child) {
+            [min, nat] = child.get_preferred_height(-1);
+
+            const [, nat2] = child.get_preferred_height(forWidth);
+            const expHeight =
+                Math.min(nat2, nat * this._expandLines);
             [min, nat] = [
-                min + this._expansion * (expMin - min),
-                nat + this._expansion * (expNat - nat),
+                min + this._expansion * (expHeight - min),
+                nat + this._expansion * (expHeight - nat),
             ];
         }
 
         return [min, nat];
     }
+});
 
-    vfunc_allocate(container, box) {
-        for (let i = 0; i < container.get_n_children(); i++) {
-            let child = container.get_child_at_index(i);
+export const Source = GObject.registerClass({
+    Properties: {
+        'title': GObject.ParamSpec.string(
+            'title', 'title', 'title',
+            GObject.ParamFlags.READWRITE,
+            null),
+        'icon': GObject.ParamSpec.object(
+            'icon', 'icon', 'icon',
+            GObject.ParamFlags.READWRITE,
+            Gio.Icon),
+        'icon-name': GObject.ParamSpec.string(
+            'icon-name', 'icon-name', 'icon-name',
+            GObject.ParamFlags.READWRITE,
+            null),
+    },
+}, class Source extends GObject.Object {
+    get iconName() {
+        if (this.gicon instanceof Gio.ThemedIcon)
+            return this.gicon.iconName;
+        else
+            return null;
+    }
 
-            if (child.visible)
-                child.allocate(box);
-        }
+    set iconName(iconName) {
+        this.icon = new Gio.ThemedIcon({name: iconName});
     }
 });
 
+const TimeLabel = GObject.registerClass(
+class TimeLabel extends St.Label {
+    _init() {
+        super._init({
+            style_class: 'event-time',
+            x_expand: true,
+            y_expand: true,
+            x_align: Clutter.ActorAlign.START,
+            y_align: Clutter.ActorAlign.END,
+            visible: false,
+        });
+    }
+
+    get datetime() {
+        return this._datetime;
+    }
+
+    set datetime(datetime) {
+        if (this._datetime?.equal(datetime))
+            return;
+
+        this._datetime = datetime;
+
+        this.visible = !!this._datetime;
+
+        if (this.mapped)
+            this._updateText();
+    }
+
+    _updateText() {
+        if (this._datetime)
+            this.text = formatTimeSpan(this._datetime);
+    }
+
+    vfunc_map() {
+        this._updateText();
+
+        super.vfunc_map();
+    }
+});
+
+const MessageHeader = GObject.registerClass(
+class MessageHeader extends St.BoxLayout {
+    constructor(source) {
+        super({
+            style_class: 'message-header',
+            x_expand: true,
+        });
+
+        const sourceIconEffect = new Clutter.DesaturateEffect();
+        const sourceIcon = new St.Icon({
+            style_class: 'message-source-icon',
+            y_align: Clutter.ActorAlign.CENTER,
+            fallback_icon_name: 'application-x-executable-symbolic',
+        });
+        sourceIcon.add_effect(sourceIconEffect);
+        this.add_child(sourceIcon);
+
+        sourceIcon.connect('style-changed', () => {
+            const themeNode = sourceIcon.get_theme_node();
+            sourceIconEffect.enabled = themeNode.get_icon_style() === St.IconStyle.SYMBOLIC;
+        });
+
+        const headerContent = new St.BoxLayout({
+            style_class: 'message-header-content',
+            y_align: Clutter.ActorAlign.CENTER,
+            x_expand: true,
+        });
+        this.add_child(headerContent);
+
+        this.expandButton = new St.Button({
+            style_class: 'message-expand-button',
+            icon_name: 'notification-expand-symbolic',
+            y_align: Clutter.ActorAlign.CENTER,
+            pivot_point: new Graphene.Point({x: 0.5, y: 0.5}),
+        });
+        this.add_child(this.expandButton);
+
+        this.closeButton = new St.Button({
+            style_class: 'message-close-button',
+            icon_name: 'window-close-symbolic',
+            y_align: Clutter.ActorAlign.CENTER,
+        });
+        this.add_child(this.closeButton);
+
+        const sourceTitle = new St.Label({
+            style_class: 'message-source-title',
+            y_align: Clutter.ActorAlign.END,
+        });
+        headerContent.add_child(sourceTitle);
+
+        source.bind_property_full('title',
+            sourceTitle,
+            'text',
+            GObject.BindingFlags.SYNC_CREATE,
+            // Translators: this is the string displayed in the header when a message
+            // source doesn't have a name
+            (bind, value) => [true, value === null || value === '' ? _('Unknown App') : value],
+            null);
+        source.bind_property('icon',
+            sourceIcon,
+            'gicon',
+            GObject.BindingFlags.SYNC_CREATE);
+
+        this.timeLabel = new TimeLabel();
+        headerContent.add_child(this.timeLabel);
+    }
+});
 
 export const Message = GObject.registerClass({
+    Properties: {
+        'title': GObject.ParamSpec.string(
+            'title', 'title', 'title',
+            GObject.ParamFlags.READWRITE,
+            null),
+        'body': GObject.ParamSpec.string(
+            'body', 'body', 'body',
+            GObject.ParamFlags.READWRITE,
+            null),
+        'use-body-markup': GObject.ParamSpec.boolean(
+            'use-body-markup', 'use-body-markup', 'use-body-markup',
+            GObject.ParamFlags.READWRITE,
+            false),
+        'icon': GObject.ParamSpec.object(
+            'icon', 'icon', 'icon',
+            GObject.ParamFlags.READWRITE,
+            Gio.Icon),
+        'datetime': GObject.ParamSpec.boxed(
+            'datetime', 'datetime', 'datetime',
+            GObject.ParamFlags.READWRITE,
+            GLib.DateTime),
+    },
     Signals: {
         'close': {},
         'expanded': {},
         'unexpanded': {},
     },
 }, class Message extends St.Button {
-    _init(title, body) {
-        super._init({
+    constructor(source) {
+        super({
             style_class: 'message',
             accessible_role: Atk.Role.NOTIFICATION,
             can_focus: true,
             x_expand: true,
-            y_expand: true,
+            y_expand: false,
         });
 
         this.expanded = false;
@@ -331,116 +456,144 @@ export const Message = GObject.registerClass({
         });
         this.set_child(vbox);
 
-        let hbox = new St.BoxLayout();
-        vbox.add_actor(hbox);
+        this._header = new MessageHeader(source);
+        vbox.add_child(this._header);
 
-        this._actionBin = new St.Widget({
+        const hbox = new St.BoxLayout({
+            style_class: 'message-box',
+        });
+        vbox.add_child(hbox);
+
+        this._actionBin = new St.Bin({
             layout_manager: new ScaleLayout(),
             visible: false,
         });
-        vbox.add_actor(this._actionBin);
+        vbox.add_child(this._actionBin);
 
-        this._iconBin = new St.Bin({
-            style_class: 'message-icon-bin',
+        this._icon = new St.Icon({
+            style_class: 'message-icon',
             y_expand: true,
             y_align: Clutter.ActorAlign.START,
             visible: false,
         });
-        hbox.add_actor(this._iconBin);
+        hbox.add_child(this._icon);
 
         const contentBox = new St.BoxLayout({
             style_class: 'message-content',
             vertical: true,
             x_expand: true,
         });
-        hbox.add_actor(contentBox);
+        hbox.add_child(contentBox);
 
         this._mediaControls = new St.BoxLayout();
-        hbox.add_actor(this._mediaControls);
+        hbox.add_child(this._mediaControls);
 
-        let titleBox = new St.BoxLayout();
-        contentBox.add_actor(titleBox);
-
-        this.titleLabel = new St.Label({style_class: 'message-title'});
-        this.setTitle(title);
-        titleBox.add_actor(this.titleLabel);
-
-        this._secondaryBin = new St.Bin({
-            style_class: 'message-secondary-bin',
-            x_expand: true, y_expand: true,
+        this.titleLabel = new St.Label({
+            style_class: 'message-title',
+            y_align: Clutter.ActorAlign.END,
         });
-        titleBox.add_actor(this._secondaryBin);
+        contentBox.add_child(this.titleLabel);
 
-        this._closeButton = new St.Button({
-            style_class: 'message-close-button',
-            icon_name: 'window-close-symbolic',
-            y_align: Clutter.ActorAlign.CENTER,
-            opacity: 0,
+        this._bodyLabel = new URLHighlighter('', true, this._useBodyMarkup);
+        this._bodyLabel.add_style_class_name('message-body');
+        this._bodyBin = new St.Bin({
+            x_expand: true,
+            layout_manager: new LabelExpanderLayout(),
+            child: this._bodyLabel,
         });
-        titleBox.add_actor(this._closeButton);
+        contentBox.add_child(this._bodyBin);
 
-        this._bodyStack = new St.Widget({x_expand: true});
-        this._bodyStack.layout_manager = new LabelExpanderLayout();
-        contentBox.add_actor(this._bodyStack);
-
-        this.bodyLabel = new URLHighlighter('', false, this._useBodyMarkup);
-        this.bodyLabel.add_style_class_name('message-body');
-        this._bodyStack.add_actor(this.bodyLabel);
-        this.setBody(body);
-
-        this._closeButton.connect('clicked', this.close.bind(this));
-        let actorHoverId = this.connect('notify::hover', this._sync.bind(this));
-        this._closeButton.connect('destroy', this.disconnect.bind(this, actorHoverId));
         this.connect('destroy', this._onDestroy.bind(this));
-        this._sync();
+
+        this._header.closeButton.connect('clicked', this.close.bind(this));
+        this._header.closeButton.visible = this.canClose();
+
+        this._header.expandButton.connect('clicked', () => {
+            if (this.expanded)
+                this.unexpand(true);
+            else
+                this.expand(true);
+        });
+        this._bodyLabel.connect('notify::allocation', this._updateExpandButton.bind(this));
+        this._updateExpandButton();
+    }
+
+    _updateExpandButton() {
+        if (!this._bodyLabel.has_allocation())
+            return;
+        const layout = this._bodyLabel.clutter_text.get_layout();
+        const canExpand = layout.is_ellipsized() || this.expanded || !!this._actionBin.child;
+        // Use opacity to not trigger a relayout
+        this._header.expandButton.opacity = canExpand ? 255 : 0;
     }
 
     close() {
         this.emit('close');
     }
 
-    setIcon(actor) {
-        this._iconBin.child = actor;
-        this._iconBin.visible = actor != null;
+    set icon(icon) {
+        this._icon.gicon = icon;
+
+        if (icon instanceof Gio.ThemedIcon)
+            this._icon.add_style_class_name('message-themed-icon');
+        else
+            this._icon.remove_style_class_name('message-themed-icon');
+
+        this._icon.visible = !!icon;
+        this.notify('icon');
     }
 
-    setSecondaryActor(actor) {
-        this._secondaryBin.child = actor;
+    get icon() {
+        return this._icon.gicon;
     }
 
-    setTitle(text) {
-        let title = text ? _fixMarkup(text.replace(/\n/g, ' '), false) : '';
+    set datetime(datetime) {
+        this._header.timeLabel.datetime = datetime;
+        this.notify('datetime');
+    }
+
+    get datetime() {
+        return this._header.timeLabel.datetime;
+    }
+
+    set title(text) {
+        this._titleText = text;
+        const title = text ? _fixMarkup(text.replace(/\n/g, ' '), false) : '';
         this.titleLabel.clutter_text.set_markup(title);
+        this.notify('title');
     }
 
-    setBody(text) {
+    get title() {
+        return this._titleText;
+    }
+
+    set body(text) {
         this._bodyText = text;
-        this.bodyLabel.setMarkup(text ? text.replace(/\n/g, ' ') : '',
+        this._bodyLabel.setMarkup(text ? text.replace(/\n/g, ' ') : '',
             this._useBodyMarkup);
-        if (this._expandedLabel)
-            this._expandedLabel.setMarkup(text, this._useBodyMarkup);
+        this.notify('body');
     }
 
-    setUseBodyMarkup(enable) {
+    get body() {
+        return this._bodyText;
+    }
+
+    set useBodyMarkup(enable) {
         if (this._useBodyMarkup === enable)
             return;
         this._useBodyMarkup = enable;
-        if (this.bodyLabel)
-            this.setBody(this._bodyText);
+        this.body = this._bodyText;
+        this.notify('use-body-markup');
+    }
+
+    get useBodyMarkup() {
+        return this._useBodyMarkup;
     }
 
     setActionArea(actor) {
-        if (actor == null) {
-            if (this._actionBin.get_n_children() > 0)
-                this._actionBin.get_child_at_index(0).destroy();
-            return;
-        }
-
-        if (this._actionBin.get_n_children() > 0)
-            throw new Error('Message already has an action area');
-
-        this._actionBin.add_actor(actor);
-        this._actionBin.visible = this.expanded;
+        this._actionBin.child = actor;
+        this._actionBin.visible = actor && this.expanded;
+        this._updateExpandButton();
     }
 
     addMediaControl(iconName, callback) {
@@ -449,40 +602,17 @@ export const Message = GObject.registerClass({
             iconName,
         });
         button.connect('clicked', callback);
-        this._mediaControls.add_actor(button);
+        this._mediaControls.add_child(button);
         return button;
     }
 
-    setExpandedBody(actor) {
-        if (actor == null) {
-            if (this._bodyStack.get_n_children() > 1)
-                this._bodyStack.get_child_at_index(1).destroy();
-            return;
-        }
-
-        if (this._bodyStack.get_n_children() > 1)
-            throw new Error('Message already has an expanded body actor');
-
-        this._bodyStack.insert_child_at_index(actor, 1);
-    }
-
-    setExpandedLines(nLines) {
-        this._bodyStack.layout_manager.expandLines = nLines;
-    }
-
     expand(animate) {
         this.expanded = true;
 
-        this._actionBin.visible = this._actionBin.get_n_children() > 0;
-
-        if (this._bodyStack.get_n_children() < 2) {
-            this._expandedLabel = new URLHighlighter(this._bodyText,
-                true, this._useBodyMarkup);
-            this.setExpandedBody(this._expandedLabel);
-        }
+        this._actionBin.visible = !!this._actionBin.child;
 
         const duration = animate ? MessageTray.ANIMATION_TIME : 0;
-        this._bodyStack.ease_property('@layout.expansion', 1, {
+        this._bodyBin.ease_property('@layout.expansion', 1, {
             progress_mode: Clutter.AnimationMode.EASE_OUT_QUAD,
             duration,
         });
@@ -494,12 +624,17 @@ export const Message = GObject.registerClass({
             mode: Clutter.AnimationMode.EASE_OUT_QUAD,
         });
 
+        this._header.expandButton.ease({
+            rotation_angle_z: 180,
+            duration,
+        });
+
         this.emit('expanded');
     }
 
     unexpand(animate) {
         const duration = animate ? MessageTray.ANIMATION_TIME : 0;
-        this._bodyStack.ease_property('@layout.expansion', 0, {
+        this._bodyBin.ease_property('@layout.expansion', 0, {
             progress_mode: Clutter.AnimationMode.EASE_OUT_QUAD,
             duration,
         });
@@ -514,6 +649,11 @@ export const Message = GObject.registerClass({
             },
         });
 
+        this._header.expandButton.ease({
+            rotation_angle_z: 0,
+            duration,
+        });
+
         this.emit('unexpanded');
     }
 
@@ -521,12 +661,6 @@ export const Message = GObject.registerClass({
         return false;
     }
 
-    _sync() {
-        let visible = this.hover && this.canClose();
-        this._closeButton.opacity = visible ? 255 : 0;
-        this._closeButton.reactive = visible;
-    }
-
     _onDestroy() {
     }
 
@@ -574,10 +708,10 @@ export const MessageListSection = GObject.registerClass({
             style_class: 'message-list-section-list',
             vertical: true,
         });
-        this.add_actor(this._list);
+        this.add_child(this._list);
 
-        this._list.connect('actor-added', this._sync.bind(this));
-        this._list.connect('actor-removed', this._sync.bind(this));
+        this._list.connect('child-added', this._sync.bind(this));
+        this._list.connect('child-removed', this._sync.bind(this));
 
         Main.sessionMode.connectObject(
             'updated', () => this._sync(), this);
