@@ -73,7 +73,7 @@ const AppMenuButton = GObject.registerClass({
         this._targetApp = null;
 
         let bin = new St.Bin({name: 'appMenu'});
-        this.add_actor(bin);
+        this.add_child(bin);
 
         this.bind_property('reactive', this, 'can-focus', 0);
         this.reactive = false;
@@ -91,7 +91,7 @@ const AppMenuButton = GObject.registerClass({
             y_align: Clutter.ActorAlign.CENTER,
         });
         this._iconBox.add_effect(iconEffect);
-        this._container.add_actor(this._iconBox);
+        this._container.add_child(this._iconBox);
 
         this._iconBox.connect('style-changed', () => {
             let themeNode = this._iconBox.get_theme_node();
@@ -102,7 +102,7 @@ const AppMenuButton = GObject.registerClass({
             y_expand: true,
             y_align: Clutter.ActorAlign.CENTER,
         });
-        this._container.add_actor(this._label);
+        this._container.add_child(this._label);
 
         this._visible = !Main.overview.visible;
         if (!this._visible)
@@ -115,7 +115,7 @@ const AppMenuButton = GObject.registerClass({
             animate: true,
             hideOnStop: true,
         });
-        this._container.add_actor(this._spinner);
+        this._container.add_child(this._spinner);
 
         let menu = new AppMenu(this);
         this.setMenu(menu);
@@ -427,14 +427,12 @@ class ActivitiesButton extends PanelMenu.Button {
 
         this.add_child(new WorkspaceIndicators());
 
-        Main.overview.connect('showing', () => {
-            this.add_style_pseudo_class('checked');
-            this.add_accessible_state(Atk.StateType.CHECKED);
-        });
-        Main.overview.connect('hiding', () => {
-            this.remove_style_pseudo_class('checked');
-            this.remove_accessible_state(Atk.StateType.CHECKED);
-        });
+        Main.overview.connectObject('showing',
+            () => this.add_style_pseudo_class('checked'),
+            this);
+        Main.overview.connectObject('hiding',
+            () => this.remove_style_pseudo_class('checked'),
+            this);
 
         this._xdndTimeOut = 0;
     }
@@ -670,16 +668,16 @@ class Panel extends St.Widget {
         this.connect('button-press-event', this._onButtonPress.bind(this));
         this.connect('touch-event', this._onTouchEvent.bind(this));
 
-        Main.overview.connect('showing', () => {
-            this.add_style_pseudo_class('overview');
-        });
-        Main.overview.connect('hiding', () => {
-            this.remove_style_pseudo_class('overview');
-        });
+        Main.overview.connectObject('showing',
+            () => this.add_style_pseudo_class('overview'),
+            this);
+        Main.overview.connectObject('hiding',
+            () => this.remove_style_pseudo_class('overview'),
+            this);
 
-        Main.layoutManager.panelBox.add(this);
+        Main.layoutManager.panelBox.add_child(this);
         Main.ctrlAltTabManager.addGroup(this,
-            _('Top Bar'), 'focus-top-bar-symbolic',
+            _('Top Bar'), 'shell-focus-top-bar-symbolic',
             {sortGroup: CtrlAltTab.SortGroup.TOP});
 
         Main.sessionMode.connect('updated', this._updatePanel.bind(this));
@@ -763,17 +761,19 @@ class Panel extends St.Widget {
         if (targetActor !== this)
             return Clutter.EVENT_PROPAGATE;
 
-        const [x, y_] = event.get_coords();
+        const [x, y] = event.get_coords();
         let dragWindow = this._getDraggableWindowForPosition(x);
 
         if (!dragWindow)
             return Clutter.EVENT_PROPAGATE;
 
+        const positionHint = new Graphene.Point({x, y});
         return dragWindow.begin_grab_op(
             Meta.GrabOp.MOVING,
             event.get_device(),
             event.get_event_sequence(),
-            event.get_time()) ? Clutter.EVENT_STOP : Clutter.EVENT_PROPAGATE;
+            event.get_time(),
+            positionHint) ? Clutter.EVENT_STOP : Clutter.EVENT_PROPAGATE;
     }
 
     _onButtonPress(actor, event) {
@@ -919,7 +919,7 @@ class Panel extends St.Widget {
 
         let parent = container.get_parent();
         if (parent)
-            parent.remove_actor(container);
+            parent.remove_child(container);
 
 
         box.insert_child_at_index(container, position);
