@@ -62,7 +62,7 @@ export const QuickToggle = GObject.registerClass({
             ...params,
         });
 
-        this._box = new St.BoxLayout();
+        this._box = new St.BoxLayout({x_expand: true});
         this.set_child(this._box);
 
         const iconProps = {};
@@ -167,7 +167,7 @@ export const QuickMenuToggle = GObject.registerClass({
 
         this.add_style_class_name('quick-menu-toggle');
 
-        this._box = new St.BoxLayout();
+        this._box = new St.BoxLayout({x_expand: true});
         this.set_child(this._box);
 
         const contents = new QuickToggle({
@@ -264,7 +264,7 @@ export const QuickSlider = GObject.registerClass({
             hasMenu: true,
         });
 
-        const box = new St.BoxLayout();
+        const box = new St.BoxLayout({x_expand: true});
         this.set_child(box);
 
         const iconProps = {};
@@ -365,7 +365,6 @@ class QuickToggleMenu extends PopupMenu.PopupMenuBase {
             reactive: true,
             x_expand: true,
             y_expand: false,
-            height: 0,
             constraints,
         });
         this.actor._delegate = this;
@@ -446,19 +445,17 @@ class QuickToggleMenu extends PopupMenu.PopupMenuBase {
         this.actor.show();
         this.isOpen = true;
 
-        const previousHeight = this.actor.height;
         this.actor.height = -1;
         const [targetHeight] = this.actor.get_preferred_height(-1);
-        this.actor.height = previousHeight;
-        const distance = Math.abs(targetHeight - previousHeight);
 
         const duration = animate !== PopupAnimation.NONE
             ? POPUP_ANIMATION_TIME / 2
             : 0;
 
+        this.actor.height = 0;
         this.box.opacity = 0;
         this.actor.ease({
-            duration: duration * (distance / targetHeight),
+            duration,
             height: targetHeight,
             onComplete: () => {
                 this.box.ease({
@@ -475,13 +472,12 @@ class QuickToggleMenu extends PopupMenu.PopupMenuBase {
         if (!this.isOpen)
             return;
 
-        const {opacity} = this.box;
         const duration = animate !== PopupAnimation.NONE
             ? POPUP_ANIMATION_TIME / 2
             : 0;
 
         this.box.ease({
-            duration: duration * (opacity / 255),
+            duration,
             opacity: 0,
             onComplete: () => {
                 this.actor.ease({
@@ -851,7 +847,7 @@ class SystemIndicator extends St.BoxLayout {
 
     _addIndicator() {
         const icon = new St.Icon({style_class: 'system-status-icon'});
-        this.add_actor(icon);
+        this.add_child(icon);
         icon.connect('notify::visible', () => this._syncIndicatorsVisible());
         this._syncIndicatorsVisible();
         return icon;
