@@ -1,14 +1,18 @@
 // -*- mode: js; js-indent-level: 4; indent-tabs-mode: nil -*-
-/* exported Indicator */
 
-const { Clutter, Gio, GLib, GObject, Shell, St } = imports.gi;
+import Clutter from 'gi://Clutter';
+import Gio from 'gi://Gio';
+import GLib from 'gi://GLib';
+import GObject from 'gi://GObject';
+import Shell from 'gi://Shell';
+import St from 'gi://St';
 
-const Dialog = imports.ui.dialog;
-const ModalDialog = imports.ui.modalDialog;
-const PermissionStore = imports.misc.permissionStore;
-const {SystemIndicator} = imports.ui.quickSettings;
+import * as Dialog from '../dialog.js';
+import * as ModalDialog from '../modalDialog.js';
+import * as PermissionStore from '../../misc/permissionStore.js';
+import {SystemIndicator} from '../quickSettings.js';
 
-const { loadInterfaceXML } = imports.misc.fileUtils;
+import {loadInterfaceXML} from '../../misc/fileUtils.js';
 
 const LOCATION_SCHEMA = 'org.gnome.system.location';
 const MAX_ACCURACY_LEVEL = 'max-accuracy-level';
@@ -17,7 +21,8 @@ const ENABLED = 'enabled';
 const APP_PERMISSIONS_TABLE = 'location';
 const APP_PERMISSIONS_ID = 'location';
 
-var GeoclueAccuracyLevel = {
+/** @enum {number} */
+const GeoclueAccuracyLevel = {
     NONE: 0,
     COUNTRY: 1,
     CITY: 4,
@@ -28,26 +33,30 @@ var GeoclueAccuracyLevel = {
 
 function accuracyLevelToString(accuracyLevel) {
     for (let key in GeoclueAccuracyLevel) {
-        if (GeoclueAccuracyLevel[key] == accuracyLevel)
+        if (GeoclueAccuracyLevel[key] === accuracyLevel)
             return key;
     }
 
     return 'NONE';
 }
 
-var GeoclueIface = loadInterfaceXML('org.freedesktop.GeoClue2.Manager');
+const GeoclueIface = loadInterfaceXML('org.freedesktop.GeoClue2.Manager');
 const GeoclueManager = Gio.DBusProxy.makeProxyWrapper(GeoclueIface);
 
-var AgentIface = loadInterfaceXML('org.freedesktop.GeoClue2.Agent');
+const AgentIface = loadInterfaceXML('org.freedesktop.GeoClue2.Agent');
 
 let _geoclueAgent = null;
-function _getGeoclueAgent() {
+
+/**
+ * @returns {GeoclueAgent} - the GeoclueAgent singleton
+ */
+export function getGeoclueAgent() {
     if (_geoclueAgent === null)
         _geoclueAgent = new GeoclueAgent();
     return _geoclueAgent;
 }
 
-var GeoclueAgent = GObject.registerClass({
+const GeoclueAgent = GObject.registerClass({
     Properties: {
         'enabled': GObject.ParamSpec.boolean(
             'enabled', 'Enabled', 'Enabled',
@@ -66,7 +75,7 @@ var GeoclueAgent = GObject.registerClass({
     _init() {
         super._init();
 
-        this._settings = new Gio.Settings({ schema_id: LOCATION_SCHEMA });
+        this._settings = new Gio.Settings({schema_id: LOCATION_SCHEMA});
         this._settings.connectObject(
             `changed::${ENABLED}`, () => this.notify('enabled'),
             `changed::${MAX_ACCURACY_LEVEL}`, () => this._onMaxAccuracyLevelChanged(),
@@ -78,10 +87,10 @@ var GeoclueAgent = GObject.registerClass({
         this.connect('notify::enabled', this._onMaxAccuracyLevelChanged.bind(this));
 
         this._watchId = Gio.bus_watch_name(Gio.BusType.SYSTEM,
-                                           'org.freedesktop.GeoClue2',
-                                           0,
-                                           this._connectToGeoclue.bind(this),
-                                           this._onGeoclueVanished.bind(this));
+            'org.freedesktop.GeoClue2',
+            0,
+            this._connectToGeoclue.bind(this),
+            this._onGeoclueVanished.bind(this));
         this._onMaxAccuracyLevelChanged();
         this._connectToGeoclue();
         this._connectToPermissionStore();
@@ -131,9 +140,9 @@ var GeoclueAgent = GObject.registerClass({
 
         this._connecting = true;
         new GeoclueManager(Gio.DBus.system,
-                           'org.freedesktop.GeoClue2',
-                           '/org/freedesktop/GeoClue2/Manager',
-                           this._onManagerProxyReady.bind(this));
+            'org.freedesktop.GeoClue2',
+            '/org/freedesktop/GeoClue2/Manager',
+            this._onManagerProxyReady.bind(this));
         return true;
     }
 
@@ -201,12 +210,12 @@ var GeoclueAgent = GObject.registerClass({
     }
 });
 
-var Indicator = GObject.registerClass(
+export const Indicator = GObject.registerClass(
 class Indicator extends SystemIndicator {
     _init() {
         super._init();
 
-        this._agent = _getGeoclueAgent();
+        this._agent = getGeoclueAgent();
 
         this._indicator = this._addIndicator();
         this._indicator.icon_name = 'location-services-active-symbolic';
@@ -217,7 +226,7 @@ class Indicator extends SystemIndicator {
     }
 });
 
-var AppAuthorizer = class {
+class AppAuthorizer {
     constructor(desktopId, reqAccuracyLevel, permStoreProxy, maxAccuracyLevel) {
         this.desktopId = desktopId;
         this.reqAccuracyLevel = reqAccuracyLevel;
@@ -270,7 +279,7 @@ var AppAuthorizer = class {
     _userAuthorizeApp() {
         let name = this._app.get_name();
         let appInfo = this._app.get_app_info();
-        let reason = appInfo.get_locale_string("X-Geoclue-Reason");
+        let reason = appInfo.get_locale_string('X-Geoclue-Reason');
 
         this._dialog =
             new GeolocationDialog(name, reason, this.reqAccuracyLevel);
@@ -287,7 +296,7 @@ var AppAuthorizer = class {
     }
 
     _completeAuth() {
-        if (this._accuracyLevel != GeoclueAccuracyLevel.NONE) {
+        if (this._accuracyLevel !== GeoclueAccuracyLevel.NONE) {
             this._accuracyLevel = Math.clamp(this._accuracyLevel,
                 0, this._maxAccuracyLevel);
         }
@@ -317,13 +326,13 @@ var AppAuthorizer = class {
             log(error.message);
         }
     }
-};
+}
 
-var GeolocationDialog = GObject.registerClass({
-    Signals: { 'response': { param_types: [GObject.TYPE_UINT] } },
+export const GeolocationDialog = GObject.registerClass({
+    Signals: {'response': {param_types: [GObject.TYPE_UINT]}},
 }, class GeolocationDialog extends ModalDialog.ModalDialog {
     _init(name, reason, reqAccuracyLevel) {
-        super._init({ styleClass: 'geolocation-dialog' });
+        super._init({styleClass: 'geolocation-dialog'});
         this.reqAccuracyLevel = reqAccuracyLevel;
 
         let content = new Dialog.MessageDialogContent({
