@@ -1,11 +1,12 @@
 // -*- mode: js; js-indent-level: 4; indent-tabs-mode: nil -*-
-/* exported Indicator */
 
-const {Gio, GLib, GObject} = imports.gi;
+import Gio from 'gi://Gio';
+import GLib from 'gi://GLib';
+import GObject from 'gi://GObject';
 
-const {QuickToggle, SystemIndicator} = imports.ui.quickSettings;
+import {QuickToggle, SystemIndicator} from '../quickSettings.js';
 
-const {loadInterfaceXML} = imports.misc.fileUtils;
+import {loadInterfaceXML} from '../../misc/fileUtils.js';
 
 const BUS_NAME = 'org.gnome.SettingsDaemon.Rfkill';
 const OBJECT_PATH = '/org/gnome/SettingsDaemon/Rfkill';
@@ -80,8 +81,12 @@ const RfkillManager = GObject.registerClass({
     }
 });
 
-var _manager;
-function getRfkillManager() {
+let _manager;
+
+/**
+ * @returns {RfkillManager}
+ */
+export function getRfkillManager() {
     if (_manager != null)
         return _manager;
 
@@ -93,7 +98,7 @@ const RfkillToggle = GObject.registerClass(
 class RfkillToggle extends QuickToggle {
     _init() {
         super._init({
-            label: _('Airplane Mode'),
+            title: _('Airplane Mode'),
             iconName: 'airplane-mode-symbolic',
         });
 
@@ -110,7 +115,7 @@ class RfkillToggle extends QuickToggle {
     }
 });
 
-var Indicator = GObject.registerClass(
+export const Indicator = GObject.registerClass(
 class Indicator extends SystemIndicator {
     _init() {
         super._init();
