@@ -37,31 +37,37 @@ export class WindowAttentionHandler {
         let source = new WindowAttentionSource(app, window);
         Main.messageTray.add(source);
 
-        let [title, banner] = this._getTitleAndBanner(app, window);
+        let [title, body] = this._getTitleAndBanner(app, window);
 
-        let notification = new MessageTray.Notification(source, title, banner);
+        let notification = new MessageTray.Notification({
+            source,
+            title,
+            body,
+            forFeedback: true,
+        });
         notification.connect('activated', () => {
             source.open();
         });
-        notification.setForFeedback(true);
 
-        source.showNotification(notification);
+        source.addNotification(notification);
 
         window.connectObject('notify::title', () => {
-            [title, banner] = this._getTitleAndBanner(app, window);
-            notification.update(title, banner);
+            [title, body] = this._getTitleAndBanner(app, window);
+            notification.set({title, body});
         }, source);
     }
 }
 
 const WindowAttentionSource = GObject.registerClass(
 class WindowAttentionSource extends MessageTray.Source {
-    _init(app, window) {
+    constructor(app, window) {
+        super({
+            title: app.get_name(),
+            icon: app.get_icon(),
+            policy: MessageTray.NotificationPolicy.newForApp(app),
+        });
+
         this._window = window;
-        this._app = app;
-
-        super._init(app.get_name());
-
         this._window.connectObject(
             'notify::demands-attention', this._sync.bind(this),
             'notify::urgent', this._sync.bind(this),
@@ -75,19 +81,6 @@ class WindowAttentionSource extends MessageTray.Source {
         this.destroy();
     }
 
-    _createPolicy() {
-        if (this._app && this._app.get_app_info()) {
-            let id = this._app.get_id().replace(/\.desktop$/, '');
-            return new MessageTray.NotificationApplicationPolicy(id);
-        } else {
-            return new MessageTray.NotificationGenericPolicy();
-        }
-    }
-
-    createIcon(size) {
-        return this._app.create_icon_texture(size);
-    }
-
     destroy(params) {
         this._window.disconnectObject(this);
 
