@@ -33,7 +33,7 @@ class WorkspaceGroup extends Clutter.Actor {
         if (this._workspace) {
             this._background = new Meta.BackgroundGroup();
 
-            this.add_actor(this._background);
+            this.add_child(this._background);
 
             this._bgManager = new Background.BackgroundManager({
                 container: this._background,
@@ -58,9 +58,6 @@ class WorkspaceGroup extends Clutter.Actor {
         if (!window.showing_on_its_workspace() || this._isDesktopWindow(window))
             return false;
 
-        if (window.is_override_redirect())
-            return false;
-
         if (!this._windowIsOnThisMonitor(window))
             return false;
 
