@@ -1,13 +1,12 @@
 // -*- mode: js; js-indent-level: 4; indent-tabs-mode: nil -*-
-/* exported XdndHandler */
 
-const { Clutter } = imports.gi;
-const Signals = imports.misc.signals;
+import Clutter from 'gi://Clutter';
+import * as Signals from '../misc/signals.js';
 
-const DND = imports.ui.dnd;
-const Main = imports.ui.main;
+import * as DND from './dnd.js';
+import * as Main from './main.js';
 
-var XdndHandler = class extends Signals.EventEmitter {
+export class XdndHandler extends Signals.EventEmitter {
     constructor() {
         super();
 
@@ -16,8 +15,8 @@ var XdndHandler = class extends Signals.EventEmitter {
         this._cursorWindowClone = null;
 
         // Used as a drag actor in case we don't have a cursor window clone
-        this._dummy = new Clutter.Actor({ width: 1, height: 1, opacity: 0 });
-        Main.uiGroup.add_actor(this._dummy);
+        this._dummy = new Clutter.Actor({width: 1, height: 1, opacity: 0});
+        Main.uiGroup.add_child(this._dummy);
         this._dummy.hide();
 
         var dnd = global.backend.get_dnd();
@@ -61,8 +60,8 @@ var XdndHandler = class extends Signals.EventEmitter {
                 source: cursorWindow,
             });
 
-            this._cursorWindowClone = new Clutter.Clone({ source: cursorWindow });
-            Main.uiGroup.add_actor(this._cursorWindowClone);
+            this._cursorWindowClone = new Clutter.Clone({source: cursorWindow});
+            Main.uiGroup.add_child(this._cursorWindowClone);
 
             // Make sure that the clone has the same position as the source
             this._cursorWindowClone.add_constraint(constraintPosition);
@@ -94,7 +93,7 @@ var XdndHandler = class extends Signals.EventEmitter {
             let motionFunc = DND.dragMonitors[i].dragMotion;
             if (motionFunc) {
                 let result = motionFunc(dragEvent);
-                if (result != DND.DragMotionResult.CONTINUE)
+                if (result !== DND.DragMotionResult.CONTINUE)
                     return;
             }
         }
@@ -103,14 +102,14 @@ var XdndHandler = class extends Signals.EventEmitter {
             if (pickedActor._delegate && pickedActor._delegate.handleDragOver) {
                 let [r_, targX, targY] = pickedActor.transform_stage_point(x, y);
                 let result = pickedActor._delegate.handleDragOver(this,
-                                                                  dragEvent.dragActor,
-                                                                  targX,
-                                                                  targY,
-                                                                  global.get_current_time());
-                if (result != DND.DragMotionResult.CONTINUE)
+                    dragEvent.dragActor,
+                    targX,
+                    targY,
+                    global.get_current_time());
+                if (result !== DND.DragMotionResult.CONTINUE)
                     return;
             }
             pickedActor = pickedActor.get_parent();
         }
     }
-};
+}
