@@ -80,7 +80,6 @@ struct _ShellApp
                           * the way shell-window-tracker.c works).
                           */
   GIcon *fallback_icon;
-  MetaWindow *fallback_icon_window;
 
   ShellAppRunningState *running_state;
 
@@ -191,35 +190,6 @@ window_backed_app_get_window (ShellApp     *app)
     return NULL;
 }
 
-static GIcon *
-x11_window_create_fallback_gicon (MetaWindow *window)
-{
-  StTextureCache *texture_cache;
-  cairo_surface_t *surface;
-
-  g_object_get (window, "icon", &surface, NULL);
-
-  texture_cache = st_texture_cache_get_default ();
-  return st_texture_cache_load_cairo_surface_to_gicon (texture_cache, surface);
-}
-
-static void
-on_window_icon_changed (GObject          *object,
-                        const GParamSpec *pspec,
-                        gpointer          user_data)
-{
-  MetaWindow *window = META_WINDOW (object);
-  ShellApp *app = user_data;
-
-  g_clear_object (&app->fallback_icon);
-  app->fallback_icon = x11_window_create_fallback_gicon (window);
-
-  if (!app->fallback_icon)
-    app->fallback_icon = g_themed_icon_new ("application-x-executable");
-
-  g_object_notify_by_pspec (G_OBJECT (app), props[PROP_ICON]);
-}
-
 /**
  * shell_app_get_icon:
  *
@@ -230,35 +200,13 @@ on_window_icon_changed (GObject          *object,
 GIcon *
 shell_app_get_icon (ShellApp *app)
 {
-  MetaWindow *window = NULL;
-
   g_return_val_if_fail (SHELL_IS_APP (app), NULL);
 
   if (app->info)
     return g_app_info_get_icon (G_APP_INFO (app->info));
 
-  if (app->fallback_icon)
-    return app->fallback_icon;
-
-  /* During a state transition from running to not-running for
-   * window-backend apps, it's possible we get a request for the icon.
-   * Avoid asserting here and just return a fallback icon
-   */
-  if (app->running_state != NULL)
-    window = window_backed_app_get_window (app);
-
-  if (window &&
-      meta_window_get_client_type (window) == META_WINDOW_CLIENT_TYPE_X11)
-    {
-      app->fallback_icon_window = window;
-      app->fallback_icon = x11_window_create_fallback_gicon (window);
-      g_signal_connect (G_OBJECT (window),
-                        "notify::icon", G_CALLBACK (on_window_icon_changed), app);
-    }
-  else
-    {
-      app->fallback_icon = g_themed_icon_new ("application-x-executable");
-    }
+  if (!app->fallback_icon)
+    app->fallback_icon = g_themed_icon_new ("application-x-executable");
 
   return app->fallback_icon;
 }
@@ -449,6 +397,20 @@ shell_app_activate_window (ShellApp     *app,
           return;
         }
 
+      /* Now raise all the other windows for the app that are on
+       * the same workspace, in reverse order to preserve the stacking.
+       */
+      windows_reversed = g_slist_copy (windows);
+      windows_reversed = g_slist_reverse (windows_reversed);
+      for (iter = windows_reversed; iter; iter = iter->next)
+        {
+          MetaWindow *other_window = iter->data;
+
+          if (other_window != window)
+            meta_window_raise_and_make_recent_on_workspace (other_window, workspace);
+        }
+      g_slist_free (windows_reversed);
+
       /* If we have a transient that the user's interacted with more recently than
        * the window, pick that.
        */
@@ -460,23 +422,10 @@ shell_app_activate_window (ShellApp     *app,
         window = most_recent_transient;
 
 
-      /* First activate the window, so that we are on the correct
-       * workspace when raising other windows
-       */
       if (active != workspace)
         meta_workspace_activate_with_focus (workspace, window, timestamp);
       else
         meta_window_activate (window, timestamp);
-
-      /* Now raise all the windows for the app in reverse order to
-       * preserve the stacking. This only has an effect for windows
-       * on the current workspace
-       */
-      windows_reversed = g_slist_copy (windows);
-      windows_reversed = g_slist_reverse (windows_reversed);
-      for (iter = windows_reversed; iter; iter = iter->next)
-        meta_window_raise_and_make_recent (META_WINDOW (iter->data));
-      g_slist_free (windows_reversed);
     }
 }
 
@@ -1169,15 +1118,6 @@ _shell_app_remove_window (ShellApp   *app,
 
   g_signal_handlers_disconnect_by_func (window, G_CALLBACK(shell_app_on_user_time_changed), app);
   g_signal_handlers_disconnect_by_func (window, G_CALLBACK(shell_app_on_skip_taskbar_changed), app);
-  if (window == app->fallback_icon_window)
-    {
-      g_signal_handlers_disconnect_by_func (window, G_CALLBACK(on_window_icon_changed), app);
-      app->fallback_icon_window = NULL;
-
-      /* Select a new icon from a different window. */
-      g_clear_object (&app->fallback_icon);
-      g_object_notify_by_pspec (G_OBJECT (app), props[PROP_ICON]);
-    }
 
   g_object_unref (window);
 
@@ -1296,8 +1236,9 @@ static void
 child_context_setup (gpointer user_data)
 {
   ShellGlobal *shell_global = user_data;
-  MetaContext *meta_context = shell_global_get_context (shell_global);
+  MetaContext *meta_context;
 
+  g_object_get (shell_global, "context", &meta_context, NULL);
   meta_context_restore_rlimit_nofile (meta_context, NULL);
 }
 
@@ -1506,6 +1447,196 @@ shell_app_launch_action (ShellApp        *app,
   g_object_unref (context);
 }
 
+static gchar *
+object_path_from_app_id (const gchar *app_id)
+{
+  gchar *app_id_path, *iter;
+
+  app_id_path = g_strconcat ("/", app_id, NULL);
+  for (iter = app_id_path; *iter; iter++)
+    {
+      if (*iter == '.')
+        *iter = '/';
+
+      if (*iter == '-')
+        *iter = '_';
+    }
+
+  return app_id_path;
+}
+
+static GVariant *
+get_platform_data (ShellApp *app,
+                   guint     timestamp,
+                   int       workspace)
+{
+  GVariantBuilder builder;
+  ShellGlobal *global;
+  g_autoptr (GAppLaunchContext) context = NULL;
+  gchar *startup_id;
+
+  g_variant_builder_init (&builder, G_VARIANT_TYPE_VARDICT);
+  if (!app->info)
+    return g_variant_builder_end (&builder);
+
+  global = shell_global_get ();
+  context = shell_global_create_app_launch_context (global, timestamp, workspace);
+
+  if (!context)
+    return g_variant_builder_end (&builder);
+
+  startup_id = g_app_launch_context_get_startup_notify_id (context, G_APP_INFO (app->info), NULL);
+
+  if (!startup_id)
+    return g_variant_builder_end (&builder);
+
+
+  g_variant_builder_add (&builder, "{sv}",
+      "desktop-startup-id", g_variant_new_string (startup_id));
+  g_variant_builder_add (&builder, "{sv}",
+      "activation-token", g_variant_new_take_string (g_steal_pointer (&startup_id)));
+
+  return g_variant_builder_end (&builder);
+}
+
+static void
+on_activate_action_cb (GObject      *source,
+                       GAsyncResult *res,
+                       gpointer      user_data)
+{
+  GTask *task = G_TASK (user_data);
+  g_autoptr (GVariant) value = NULL;
+  g_autoptr (GError) error = NULL;
+
+  value = g_dbus_connection_call_finish (G_DBUS_CONNECTION (source),
+                                         res, &error);
+
+  if (error)
+      g_task_return_error (task, g_steal_pointer (&error));
+  else
+      g_task_return_boolean (task, TRUE);
+}
+
+static void
+activate_action_get_bus_cb (GObject      *object,
+                            GAsyncResult *result,
+                            gpointer      user_data)
+{
+  GTask *task = G_TASK (user_data);
+  ShellApp *app = NULL;
+  g_autoptr (GDBusConnection) session_bus = NULL;
+  g_autoptr (GError) error = NULL;
+  g_autofree gchar *object_path = NULL;
+  g_autofree gchar *app_id = NULL;
+  gchar *last_dot;
+
+  session_bus = g_bus_get_finish (result, &error);
+
+  if (error)
+    {
+      g_task_return_error (task, g_steal_pointer (&error));
+      return;
+    }
+
+  app = SHELL_APP (g_task_get_source_object (task));
+
+  app_id = g_strdup (g_app_info_get_id (G_APP_INFO (app->info)));
+  last_dot = strrchr (app_id, '.');
+  if (last_dot && g_str_equal (last_dot, ".desktop"))
+      *last_dot = '\0';
+
+  object_path = object_path_from_app_id (app_id);
+
+  g_dbus_connection_call (session_bus,
+                           app_id, object_path,
+                           "org.freedesktop.Application", "ActivateAction",
+                           g_task_get_task_data (task),
+                           NULL, G_DBUS_CALL_FLAGS_NONE, -1,
+                           g_task_get_cancellable (task),
+                           on_activate_action_cb, task);
+}
+
+
+/**
+ * shell_app_activate_action
+ * @app: the #ShellApp
+ * @action_name: the name of an action to activate
+ * @parameter: (nullable): the parameter to the activation
+ * @timestamp: Event timestamp, or 0 for current event timestamp
+ * @workspace: Start on this workspace, or -1 for default
+ * @cancellable: (nullable): a #GCancellable or %NULL
+ * @callback: (scope async): A #GAsyncReadyCallback to call when the request is satisfied.
+ * @user_data: (nullable): User data to pass to @callback
+ *
+ * This activates an action using 'org.freedesktop.Application' DBus interface.
+ *
+ * This function will fail if this #ShellApp doesn't have a valid #GDesktopAppInfo
+ * with a valid id.
+ */
+void
+shell_app_activate_action (ShellApp            *app,
+                           const char          *action_name,
+                           GVariant            *parameter,
+                           guint                timestamp,
+                           int                  workspace,
+                           GCancellable        *cancellable,
+                           GAsyncReadyCallback  callback,
+                           gpointer             user_data)
+{
+  g_autoptr (GTask) task = NULL;
+  g_autoptr (GVariant) task_data = NULL;
+
+  g_return_if_fail (SHELL_IS_APP (app));
+  g_return_if_fail (G_IS_DESKTOP_APP_INFO (app->info));
+  g_return_if_fail (g_application_id_is_valid (g_app_info_get_id (G_APP_INFO (app->info))));
+  g_return_if_fail (action_name != NULL && action_name[0] != '\0');
+  g_return_if_fail (parameter == NULL || g_variant_is_of_type (parameter,  G_VARIANT_TYPE ("av")));
+  g_return_if_fail (timestamp >= 0);
+  g_return_if_fail (workspace >= -1);
+  g_return_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable));
+
+  task = g_task_new (app, cancellable, callback, user_data);
+  g_task_set_source_tag (task, shell_app_activate_action);
+
+  if (!parameter)
+    parameter = g_variant_new("av", NULL);
+
+  task_data = g_variant_ref_sink (g_variant_new ("(s@av@a{sv})", action_name, parameter,
+                                                 get_platform_data (app, timestamp, workspace)));
+
+  g_task_set_task_data (task, g_steal_pointer (&task_data), (GDestroyNotify) g_variant_unref);
+
+  g_bus_get (G_BUS_TYPE_SESSION, cancellable, activate_action_get_bus_cb, g_steal_pointer (&task));
+}
+
+/**
+ * shell_app_activate_action_finish:
+ * @app: the #ShellApp
+ * @error: #GError for error reporting
+ *
+ * Finish the asynchronous operation started by shell_app_activate_action()
+ * and obtain its result.
+ *
+ * Returns: whether the operation was successful
+ *
+ */
+gboolean
+shell_app_activate_action_finish (ShellApp      *app,
+                                  GAsyncResult  *result,
+                                  GError       **error)
+{
+  g_return_val_if_fail (SHELL_IS_APP (app), FALSE);
+  g_return_val_if_fail (G_IS_TASK (result), FALSE);
+  g_return_val_if_fail (g_async_result_is_tagged (result,
+                                                  shell_app_activate_action),
+                        FALSE);
+
+  if (!g_task_propagate_boolean (G_TASK (result), error))
+    return FALSE;
+
+  return TRUE;
+}
+
 /**
  * shell_app_get_app_info:
  * @app: a #ShellApp
