@@ -19,9 +19,6 @@ GjsContext *_shell_global_get_gjs_context (ShellGlobal  *global);
 
 ShellAppCache * shell_global_get_app_cache (ShellGlobal *global);
 
-gboolean _shell_global_check_xdnd_event (ShellGlobal  *global,
-                                         XEvent       *xev);
-
 void _shell_global_locate_pointer (ShellGlobal  *global);
 
 void _shell_global_notify_shutdown (ShellGlobal *global);
