@@ -313,7 +313,7 @@ get_app_from_window_group (ShellWindowTracker  *tracker,
   MetaGroup *group;
   GSList *iter;
 
-  group = meta_window_get_group (window);
+  group = meta_window_x11_get_group (window);
   if (group == NULL)
     return NULL;
 
@@ -412,13 +412,6 @@ get_app_for_window (ShellWindowTracker    *tracker,
   if (meta_window_is_remote (window))
     return _shell_app_new_for_window (window);
 
-  /* HACK: Exclude screen recorder from tracking until
-   * we implement a proper fix
-   */
-  if (meta_window_get_wm_class (window) == NULL &&
-      g_strcmp0 (meta_window_get_title (window), "OpenGL Renderer") == 0)
-    return NULL;
-
   /* Check if the app's WM_CLASS specifies an app; this is
    * canonical if it does.
    */
@@ -470,7 +463,7 @@ get_app_for_window (ShellWindowTracker    *tracker,
   /* If we didn't get a startup-notification match, see if we matched
    * any other windows in the group.
    */
-  if (result == NULL)
+  if (result == NULL && meta_window_get_client_type (window) == META_WINDOW_CLIENT_TYPE_X11)
     result = get_app_from_window_group (tracker, window);
 
   /* Our last resort - we create a fake app from the window */
