@@ -55,157 +55,57 @@ enum
 
 static GParamSpec *props[N_PROPS] = { NULL, };
 
-static void clutter_container_iface_init (ClutterContainerIface *iface);
-
-G_DEFINE_TYPE_WITH_CODE (StBin, st_bin, ST_TYPE_WIDGET,
-                         G_ADD_PRIVATE (StBin)
-                         G_IMPLEMENT_INTERFACE (CLUTTER_TYPE_CONTAINER,
-                                                clutter_container_iface_init));
+G_DEFINE_TYPE_WITH_PRIVATE (StBin, st_bin, ST_TYPE_WIDGET)
 
 static void
-st_bin_add (ClutterContainer *container,
-            ClutterActor     *actor)
+st_bin_dispose (GObject *object)
 {
-  st_bin_set_child (ST_BIN (container), actor);
+  StBinPrivate *priv = st_bin_get_instance_private (ST_BIN (object));
+
+  g_clear_weak_pointer (&priv->child);
+
+  G_OBJECT_CLASS (st_bin_parent_class)->dispose (object);
 }
 
 static void
-st_bin_remove (ClutterContainer *container,
-               ClutterActor     *actor)
+set_child (StBin *bin, ClutterActor *child)
+{
+  StBinPrivate *priv = st_bin_get_instance_private (bin);
+
+  if (!g_set_weak_pointer (&priv->child, child))
+    return;
+
+  clutter_actor_queue_relayout (CLUTTER_ACTOR (bin));
+
+  g_object_notify_by_pspec (G_OBJECT (bin), props[PROP_CHILD]);
+}
+
+static void
+st_bin_child_added (ClutterActor *container,
+                    ClutterActor *actor)
+{
+  StBin *bin = ST_BIN (container);
+  StBinPrivate *priv = st_bin_get_instance_private (bin);
+
+  if (priv->child)
+    g_warning ("Attempting to add an actor of type %s to "
+               "an StBin, but the bin already contains a %s. "
+               "Was add_child() used repeatedly?",
+               G_OBJECT_TYPE_NAME (actor),
+               G_OBJECT_TYPE_NAME (priv->child));
+
+  set_child (ST_BIN (container), actor);
+}
+
+static void
+st_bin_child_removed (ClutterActor *container,
+                      ClutterActor *actor)
 {
   StBin *bin = ST_BIN (container);
   StBinPrivate *priv = st_bin_get_instance_private (bin);
 
   if (priv->child == actor)
-    st_bin_set_child (bin, NULL);
-}
-
-static void
-clutter_container_iface_init (ClutterContainerIface *iface)
-{
-  iface->add = st_bin_add;
-  iface->remove = st_bin_remove;
-}
-
-static double
-get_align_factor (ClutterActorAlign align)
-{
-  switch (align)
-    {
-    case CLUTTER_ACTOR_ALIGN_CENTER:
-      return 0.5;
-
-    case CLUTTER_ACTOR_ALIGN_START:
-      return 0.0;
-
-    case CLUTTER_ACTOR_ALIGN_END:
-      return 1.0;
-
-    case CLUTTER_ACTOR_ALIGN_FILL:
-      break;
-   }
-
-  return 0.0;
-}
-
-static void
-st_bin_allocate (ClutterActor          *self,
-                 const ClutterActorBox *box)
-{
-  StBinPrivate *priv = st_bin_get_instance_private (ST_BIN (self));
-
-  clutter_actor_set_allocation (self, box);
-
-  if (priv->child && clutter_actor_is_visible (priv->child))
-    {
-      StThemeNode *theme_node = st_widget_get_theme_node (ST_WIDGET (self));
-      ClutterActorAlign x_align = clutter_actor_get_x_align (priv->child);
-      ClutterActorAlign y_align = clutter_actor_get_y_align (priv->child);
-      ClutterActorBox childbox;
-
-      st_theme_node_get_content_box (theme_node, box, &childbox);
-      clutter_actor_allocate_align_fill (priv->child, &childbox,
-                                         get_align_factor (x_align),
-                                         get_align_factor (y_align),
-                                         x_align == CLUTTER_ACTOR_ALIGN_FILL,
-                                         y_align == CLUTTER_ACTOR_ALIGN_FILL);
-    }
-}
-
-static void
-st_bin_get_preferred_width (ClutterActor *self,
-                            gfloat        for_height,
-                            gfloat       *min_width_p,
-                            gfloat       *natural_width_p)
-{
-  StBinPrivate *priv = st_bin_get_instance_private (ST_BIN (self));
-  StThemeNode *theme_node = st_widget_get_theme_node (ST_WIDGET (self));
-
-  st_theme_node_adjust_for_height (theme_node, &for_height);
-
-  if (priv->child == NULL || !clutter_actor_is_visible (priv->child))
-    {
-      if (min_width_p)
-        *min_width_p = 0;
-
-      if (natural_width_p)
-        *natural_width_p = 0;
-    }
-  else
-    {
-      ClutterActorAlign y_align = clutter_actor_get_y_align (priv->child);
-
-      _st_actor_get_preferred_width (priv->child, for_height,
-                                     y_align == CLUTTER_ACTOR_ALIGN_FILL,
-                                     min_width_p,
-                                     natural_width_p);
-    }
-
-  st_theme_node_adjust_preferred_width (theme_node, min_width_p, natural_width_p);
-}
-
-static void
-st_bin_get_preferred_height (ClutterActor *self,
-                             gfloat        for_width,
-                             gfloat       *min_height_p,
-                             gfloat       *natural_height_p)
-{
-  StBinPrivate *priv = st_bin_get_instance_private (ST_BIN (self));
-  StThemeNode *theme_node = st_widget_get_theme_node (ST_WIDGET (self));
-
-  st_theme_node_adjust_for_width (theme_node, &for_width);
-
-  if (priv->child == NULL || !clutter_actor_is_visible (priv->child))
-    {
-      if (min_height_p)
-        *min_height_p = 0;
-
-      if (natural_height_p)
-        *natural_height_p = 0;
-    }
-  else
-    {
-      ClutterActorAlign x_align = clutter_actor_get_x_align (priv->child);
-
-      _st_actor_get_preferred_height (priv->child, for_width,
-                                      x_align == CLUTTER_ACTOR_ALIGN_FILL,
-                                      min_height_p,
-                                      natural_height_p);
-    }
-
-  st_theme_node_adjust_preferred_height (theme_node, min_height_p, natural_height_p);
-}
-
-static void
-st_bin_destroy (ClutterActor *actor)
-{
-  StBinPrivate *priv = st_bin_get_instance_private (ST_BIN (actor));
-
-  if (priv->child)
-    clutter_actor_destroy (priv->child);
-  g_assert (priv->child == NULL);
-
-  CLUTTER_ACTOR_CLASS (st_bin_parent_class)->destroy (actor);
+    set_child (bin, NULL);
 }
 
 static void
@@ -291,13 +191,12 @@ st_bin_class_init (StBinClass *klass)
   ClutterActorClass *actor_class = CLUTTER_ACTOR_CLASS (klass);
   StWidgetClass *widget_class = ST_WIDGET_CLASS (klass);
 
+  gobject_class->dispose = st_bin_dispose;
   gobject_class->set_property = st_bin_set_property;
   gobject_class->get_property = st_bin_get_property;
 
-  actor_class->get_preferred_width = st_bin_get_preferred_width;
-  actor_class->get_preferred_height = st_bin_get_preferred_height;
-  actor_class->allocate = st_bin_allocate;
-  actor_class->destroy = st_bin_destroy;
+  actor_class->child_added = st_bin_child_added;
+  actor_class->child_removed = st_bin_child_removed;
 
   widget_class->popup_menu = st_bin_popup_menu;
   widget_class->navigate_focus = st_bin_navigate_focus;
@@ -315,6 +214,8 @@ st_bin_class_init (StBinClass *klass)
                          ST_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY);
 
   g_object_class_install_properties (gobject_class, N_PROPS, props);
+
+  clutter_actor_class_set_layout_manager_type (actor_class, CLUTTER_TYPE_BIN_LAYOUT);
 }
 
 static void
@@ -355,36 +256,15 @@ st_bin_set_child (StBin        *bin,
 
   priv = st_bin_get_instance_private (bin);
 
-  if (priv->child == child)
-    return;
-
-  if (child)
-    {
-      ClutterActor *parent = clutter_actor_get_parent (child);
-
-      if (parent)
-        {
-          g_warning ("%s: The provided 'child' actor %p already has a "
-                     "(different) parent %p and can't be made a child of %p.",
-                     G_STRFUNC, child, parent, bin);
-          return;
-        }
-    }
+  g_object_freeze_notify (G_OBJECT (bin));
 
   if (priv->child)
     clutter_actor_remove_child (CLUTTER_ACTOR (bin), priv->child);
 
-  priv->child = NULL;
-
   if (child)
-    {
-      priv->child = child;
-      clutter_actor_add_child (CLUTTER_ACTOR (bin), child);
-    }
+    clutter_actor_add_child (CLUTTER_ACTOR (bin), child);
 
-  clutter_actor_queue_relayout (CLUTTER_ACTOR (bin));
-
-  g_object_notify_by_pspec (G_OBJECT (bin), props[PROP_CHILD]);
+  g_object_thaw_notify (G_OBJECT (bin));
 }
 
 /**
@@ -398,7 +278,11 @@ st_bin_set_child (StBin        *bin,
 ClutterActor *
 st_bin_get_child (StBin *bin)
 {
+  StBinPrivate *priv;
+
   g_return_val_if_fail (ST_IS_BIN (bin), NULL);
 
-  return ((StBinPrivate *)st_bin_get_instance_private (bin))->child;
+  priv = st_bin_get_instance_private (bin);
+
+  return priv->child;
 }
