@@ -185,7 +185,7 @@ st_icon_dispose (GObject *gobject)
   g_clear_object (&priv->gicon);
   g_clear_object (&priv->fallback_gicon);
   g_clear_pointer (&priv->colors, st_icon_colors_unref);
-  g_clear_pointer (&priv->shadow_pipeline, cogl_object_unref);
+  g_clear_object (&priv->shadow_pipeline);
   g_clear_pointer (&priv->shadow_spec, st_shadow_unref);
 
   G_OBJECT_CLASS (st_icon_parent_class)->dispose (gobject);
@@ -373,8 +373,7 @@ st_icon_init (StIcon *self)
 
   self->priv = st_icon_get_instance_private (self);
 
-  layout_manager = clutter_bin_layout_new (CLUTTER_BIN_ALIGNMENT_FILL,
-                                           CLUTTER_BIN_ALIGNMENT_FILL);
+  layout_manager = clutter_bin_layout_new ();
   clutter_actor_set_layout_manager (CLUTTER_ACTOR (self), layout_manager);
 
   texture_cache = st_texture_cache_get_default ();
@@ -395,7 +394,7 @@ st_icon_clear_shadow_pipeline (StIcon *icon)
 {
   StIconPrivate *priv = icon->priv;
 
-  g_clear_pointer (&priv->shadow_pipeline, cogl_object_unref);
+  g_clear_object (&priv->shadow_pipeline);
   graphene_size_init (&priv->shadow_size, 0, 0);
 }
 
