@@ -46,9 +46,13 @@ struct _StThemeContext {
 enum
 {
   PROP_0,
-  PROP_SCALE_FACTOR
+  PROP_SCALE_FACTOR,
+
+  N_PROPS
 };
 
+static GParamSpec *props[N_PROPS] = { NULL, };
+
 enum
 {
   CHANGED,
@@ -77,6 +81,19 @@ static void st_theme_context_get_property (GObject      *object,
                                            GValue       *value,
                                            GParamSpec   *pspec);
 
+static void
+st_theme_context_set_scale_factor (StThemeContext *context,
+                                   int             scale_factor)
+{
+  if (scale_factor == context->scale_factor)
+    return;
+
+  context->scale_factor = scale_factor;
+  g_object_notify_by_pspec (G_OBJECT (context), props[PROP_SCALE_FACTOR]);
+  st_theme_context_changed (context);
+}
+
+
 static void
 st_theme_context_finalize (GObject *object)
 {
@@ -120,13 +137,14 @@ st_theme_context_class_init (StThemeContextClass *klass)
    *
    * The scaling factor used for HiDPI scaling.
    */
-  g_object_class_install_property (object_class,
-                                   PROP_SCALE_FACTOR,
-                                   g_param_spec_int ("scale-factor",
-                                                     "Scale factor",
-                                                     "Integer scale factor used for HiDPI scaling",
-                                                     0, G_MAXINT, 1,
-                                                     ST_PARAM_READWRITE));
+  props[PROP_SCALE_FACTOR] =
+    g_param_spec_int ("scale-factor",
+                      "Scale factor",
+                      "Integer scale factor used for HiDPI scaling",
+                      0, G_MAXINT, 1,
+                      ST_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY);
+
+  g_object_class_install_properties (object_class, N_PROPS, props);
 
   /**
    * StThemeContext::changed:
@@ -179,16 +197,8 @@ st_theme_context_set_property (GObject      *object,
   switch (prop_id)
     {
     case PROP_SCALE_FACTOR:
-      {
-        int scale_factor = g_value_get_int (value);
-        if (scale_factor != context->scale_factor)
-          {
-            context->scale_factor = scale_factor;
-            st_theme_context_changed (context);
-          }
-
-        break;
-      }
+      st_theme_context_set_scale_factor (context, g_value_get_int (value));
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -333,7 +343,7 @@ st_theme_context_get_for_stage (ClutterStage *stage)
  *
  * Sets the default set of theme stylesheets for the context. This theme will
  * be used for the root node and for nodes descending from it, unless some other
- * style is explicitely specified.
+ * style is explicitly specified.
  */
 void
 st_theme_context_set_theme (StThemeContext          *context,
@@ -385,7 +395,7 @@ st_theme_context_get_theme (StThemeContext *context)
  *
  * Sets the default font for the theme context. This is the font that
  * is inherited by the root node of the tree of theme nodes. If the
- * font is not overriden, then this font will be used. If the font is
+ * font is not overridden, then this font will be used. If the font is
  * partially modified (for example, with 'font-size: 110%'), then that
  * modification is based on this font.
  */
