@@ -405,7 +405,7 @@ st_widget_allocate (ClutterActor          *actor,
   /* If we've chained up to here, we want to allocate the children using the
    * currently installed layout manager */
   clutter_layout_manager_allocate (clutter_actor_get_layout_manager (actor),
-                                   CLUTTER_CONTAINER (actor),
+                                   actor,
                                    &content_box);
 }
 
@@ -829,7 +829,7 @@ st_widget_get_paint_volume (ClutterActor *self,
            child != NULL;
            child = clutter_actor_get_next_sibling (child))
         {
-          const ClutterPaintVolume *child_volume;
+          g_autoptr (ClutterPaintVolume) child_volume = NULL;
 
           if (!clutter_actor_is_visible (child))
             continue;
