@@ -126,7 +126,8 @@ shew_window_exporter_export_finish (ShewWindowExporter  *exporter,
 }
 
 void
-shew_window_exporter_unexport (ShewWindowExporter *exporter)
+shew_window_exporter_unexport (ShewWindowExporter *exporter,
+                               const char         *handle)
 {
   GtkWidget *widget;
 
@@ -138,7 +139,11 @@ shew_window_exporter_unexport (ShewWindowExporter *exporter)
   if (GDK_IS_WAYLAND_DISPLAY (gtk_widget_get_display (widget)))
     {
       GdkSurface *s = gtk_native_get_surface (GTK_NATIVE (widget));
+#if GTK_CHECK_VERSION (4, 11, 3)
+      gdk_wayland_toplevel_drop_exported_handle (GDK_WAYLAND_TOPLEVEL (s), handle);
+#else
       gdk_wayland_toplevel_unexport_handle (GDK_WAYLAND_TOPLEVEL (s));
+#endif
     }
 #endif
 }
